
import subprocess
import os
import sys
import errno

import yaml

from viman import vimanYamlWrapper
from viman import vimanUtils

class vimanGitWrapper():
    '''
    @brief low operation wrapper of git , install , upgrade , remove , query 
    @note all other operation base on this
    '''

    DIR_PLUGIN = os.path.join(os.getenv('HOME'),'.vim/bundle')

    @staticmethod
    @vimanYamlWrapper.vimanYamlWrapper.installWrapper
    def install(url,recipe):
        '''
        @brief install a vim plugin to .vim/bundle by git from url
        @param url url of git repository
        '''
        if not os.path.isdir(vimanGitWrapper.DIR_PLUGIN):
            #print('error:don\'t exist directory `{}`!'.format(vimanGitWrapper.DIR_PLUGIN),
                    #file=sys.stderr)
            #sys.exit(errno.ENOENT)
            os.makedirs(vimanGitWrapper.DIR_PLUGIN)
        os.chdir(vimanGitWrapper.DIR_PLUGIN)
        #sys.exit(subprocess.run(['git','clone',url]))
        ret = subprocess.run(['git','clone',url])
        if not 0 == ret.returncode:
            print('error:subprocess run `{}` failed!'.format(' '.join(ret.args)),
                    file=sys.stderr)
            sys.exit(ret.returncode)
        ret = os.system(recipe)
        return ret

    @staticmethod
    def installByYml(ymlName):
        '''
        @brief install plugins by yml
        @param ymlName yml file name
        '''
        if not os.path.isfile(ymlName):
            print('error:don\'t exist file `{}`!'.format(ymlName),file=sys.stderr)
            sys.exit(errno.EINVAL)
        with open(ymlName) as f:
            yml = yaml.load(f)
            for v in yml.values():
                vimanGitWrapper.install(v['url'],v['recipe'])

    @staticmethod
    def installByCurrent(ymlString):
        if ymlString:
            yml = yaml.load(ymlString)
            if yml:
                for v in yml.values():
                    vimanGitWrapper.install(v['url'],v['recipe'])

    @staticmethod
    def upgrade(url):
        '''
        @brief upgrade a vim plugin by git from url
        @param url url of git repository
        '''
        return vimanGitWrapper.upgradeByName(vimanUtils.vimanUtils.getPlugin4Url(url))
        
    @staticmethod
    def upgradeByName(name):
        path = os.path.join(vimanGitWrapper.DIR_PLUGIN,name)
        if not os.path.isdir(path):
            print('error:don\'t exist directory `{}`!'.format(path),file=sys.stderr)
            sys.exit(errno.ENOENT)
        os.chdir(path)
        #sys.exit(subprocess.run(['git','pull']))
        ret = subprocess.run(['git','pull'])
        if not 0 == ret.returncode:
            print('error:subprocess run `{}` failed!'.format(' '.join(ret.args)),
                    file=sys.stderr)
            sys.exit(ret.returncode)
        return ret

    @staticmethod
    def upgradeByYml(ymlName):
        if not os.path.isfile(ymlName):
            print('error:don\'t exist file `{}`!'.format(ymlName),file=sys.stderr)
            sys.exit(errno.EINVAL)
        with open(ymlName) as f:
            yml = yaml.load(f)
            for v in yml.values():
                vimanGitWrapper.upgrade(v['url'])


    @staticmethod
    #@vimanYamlWrapper.vimanYamlWrapper.removeWrapper anti-multi YAML decorator
    def remove(url):
        '''
        @brief remove a vim plugin from url
        @param url url of git repository
        '''
        return vimanGitWrapper.removeByName(vimanUtils.vimanUtils.getPlugin4Url(url))

    @staticmethod
    @vimanYamlWrapper.vimanYamlWrapper.removeByNameWrapper
    def removeByName(name):
        path = os.path.join(vimanGitWrapper.DIR_PLUGIN,name)
        if not os.path.isdir(path):
            print('error:don\'t exist directory `{}`!'.format(path),file=sys.stderr)
            sys.exit(errno.ENOENT)
        #sys.exit(subprocess.run(['rm','-rf',path]))
        ret = subprocess.run(['rm','-rf',path])
        if not 0 == ret.returncode:
            print('error:subprocess run `{}` failed!'.format(' '.join(ret.args)),
                    file=sys.stderr)
            sys.exit(ret.returncode)
        return ret

    @staticmethod
    def removeByYml(ymlName):
        '''
        @brief remove plugins by yml
        @param ymlName yml file name
        '''
        if not os.path.isfile(ymlName):
            print('error:don\'t exist file `{}`!'.format(ymlName),file=sys.stderr)
            sys.exit(errno.EINVAL)
        with open(ymlName) as f:
            yml = yaml.load(f)
            for v in yml.values():
                vimanGitWrapper.remove(v['url'])





#! /usr/bin/env python3

import yaml
import os
import sys

from viman import vimanUtils

class vimanYamlWrapper():

    #ymlDefault = '~/.viman.yml'
    ymlDefault = os.path.join(os.getenv('HOME'),'.viman.yml')
    #ymlMode = 'r+'

    @staticmethod
    def installWrapper(install):
        def wrapper(url,recipe):
            ret = install(url,recipe) ## install plugin
            '''
            with open(vimanYamlWrapper.ymlDefault,'a+') as f:
                yaml.dump({vimanUtils.vimanUtils.getPlugin4Url(url):{'url':url,'recipe':''}}
                        ,stream=f,default_flow_style=False)
            '''
            '''
            with open(vimanYamlWrapper.ymlDefault,vimanYamlWrapper.ymlMode) as f:
                yml = dict(yaml.load(f))
                yml[vimanUtils.vimanUtils.getPlugin4Url(url)] = {'url':url,'recipe':''}
                f.seek(0)
                f.truncate()
                yaml.dump(yml,stream=f,default_flow_style=False)
            '''
            yml = vimanYamlWrapper.loadYml()
            if None == yml:
                yml = {}
            yml[vimanUtils.vimanUtils.getPlugin4Url(url)] = {'url':url,'recipe':recipe}
            vimanYamlWrapper.dumpYml(yml)
            return ret

        return wrapper

    @staticmethod
    def removeWrapper(remove):
        def wrapper(url):
            ret = remove(url)  ## remove plugin
            '''
            with open(vimanYamlWrapper.ymlDefault,vimanYamlWrapper.ymlMode) as f:
                yml = yaml.load(f)
                yml.pop(vimanUtils.vimanUtils.getPlugin4Url(url))
                f.seek(0)
                f.truncate()
                yaml.dump(yml,stream=f,default_flow_style=False)
            '''
            yml = vimanYamlWrapper.loadYml()
            yml.pop(vimanUtils.vimanUtils.getPlugin4Url(url))
            vimanYamlWrapper.dumpYml(yml)
            return ret
        
        return wrapper
        
    @staticmethod
    def removeByNameWrapper(removeByName):
        def wrapper(name):
            ret = removeByName(name) ## remove plugin by name
            '''
            with open(vimanYamlWrapper.ymlDefault,vimanYamlWrapper.ymlMode) as f:
                yml = yaml.load(f)
                yml.pop(name)
                f.seek(0)
                f.truncate()
                yaml.dump(yml,stream=f,default_flow_style=False)
            '''
            yml = vimanYamlWrapper.loadYml()
            yml.pop(name)
            vimanYamlWrapper.dumpYml(yml)
            return ret

        return wrapper

    @staticmethod
    def loadYml(file=ymlDefault):
        if not os.path.isfile(file):
            os.mknod(file);
        with open(file,'r') as f:
            yml = yaml.load(f)
            f.close()
            return yml

    @staticmethod
    def dumpYml(yml,file=ymlDefault):
        with open(file,'w') as f:
            yaml.dump(yml,stream=f,default_flow_style=False)
            f.close()


def _test():
    with open(vimanYamlWrapper.ymlDefault,'r') as f:
        #yaml.dump(yaml.load(f),stream = f, default_flow_style=False)
        yml = yaml.load(f)
        print(yml)

if '__main__' == __name__:
    _test()




#! /usr/bin/env python3

import argparse
import sys
import subprocess
import os
import errno

from viman import vimanArgParser
from viman import vimanGitWrapper
from viman import vimanYamlWrapper
from viman.__init__ import __version__
from viman.__init__ import __program__

PROGRAM = __program__
VERSION = __version__

#errno = ['OK',                          #0
        #'~/.vim/bundle don\'t exists!'] #1
'''
class vimanArgParser(argparse.ArgumentParser):
    def __init__(self,*args,**kwargs):
        super().__init__(...)
        # base option
        # Synchronize and option
        self.add_argument('-S','--synchronize',help='Synchronize plugin!')
        self.add_argument('-u','--sysupgrade',action='store_true',help='Upgrade all plugin!')

        #Remove and option
        self.add_argument('-R','--remove',help='Remove plugin!')

        #Upgrade and option
        self.add_argument('-U','--upgrade',help='Upgrade local plugin!')

        #Query and option
        self.add_argument('-Q','--query',help='Query local plugin!')

        #Version
        self.add_argument('-V','--version',action='version',version='%(prog)s 0.0.1')
'''


def main():
    #parse argumenits
    #argparser = vimanArgParser(prog='viman')
    #args = argparser.parse_args()
    '''
    if args.synchronize :
        vimanGitWrapper.vimanGitWrapper.install(args.synchronize)

    elif args.remove :
        vimanGitWrapper.vimanGitWrapper.remove(args.remove)
    elif args.upgrade:
        vimanGitWrapper.vimanGitWrapper.upgrade(args.upgrade)
    '''
    parser = vimanArgParser.vimanArgParser(sys.argv[1:])
    #print(parser.operations)
    #print(parser.options)
    #print(parser.targets)

    if parser.operations[0] == vimanArgParser.vimanOperations.operations['sync'][0]:
        # sync
        if vimanArgParser.vimanOptions.options['file'][0] in parser.options:
            # yml file
            for f in parser.targets:
                vimanGitWrapper.vimanGitWrapper.installByYml(f)
        elif vimanArgParser.vimanOptions.options['current'][0] in parser.options:
            # current yml string
            for yml in parser.targets:
                vimanGitWrapper.vimanGitWrapper.installByCurrent(yml)
        elif vimanArgParser.vimanOptions.options['sysupgrade'][0] in parser.options:
            # sysupgrade
            vimanGitWrapper.vimanGitWrapper.upgradeByYml(vimanYamlWrapper.vimanYamlWrapper.ymlDefault)

        else:
            for target in parser.targets:
                vimanGitWrapper.vimanGitWrapper.install(target, '')

    elif parser.operations[0] == vimanArgParser.vimanOperations.operations['remove'][0]:
        if vimanArgParser.vimanOptions.options['file'][0] in parser.options:
            for f in parser.targets:
                vimanGitWrapper.vimanGitWrapper.removeByYml(f)
        elif vimanArgParser.vimanOptions.options['name'][0] in parser.options:
            for name in parser.targets:
                vimanGitWrapper.vimanGitWrapper.removeByName(name)

        else:
            for target in parser.targets:
                vimanGitWrapper.vimanGitWrapper.remove(target)

    elif parser.operations[0] == vimanArgParser.vimanOperations.operations['upgrade'][0]:
        if vimanArgParser.vimanOptions.options['file'][0] in parser.options:
            # upgrade by yml file
            for f in parser.targets:
                vimanGitWrapper.vimanGitWrapper.upgradeByYml(f)
        elif vimanArgParser.vimanOptions.options['recursive'][0] in parser.options:
            # upgrade recursive
            vimanGitWrapper.vimanGitWrapper.upgradeByYml(vimanYamlWrapper.vimanYamlWrapper.ymlDefault)
        for target in parser.targets:
            vimanGitWrapper.vimanGitWrapper.upgrade(target)
    elif parser.operations[0] == vimanArgParser.vimanOperations.operations['query'][0]:
        yml = vimanYamlWrapper.vimanYamlWrapper.loadYml()
        if None == yml:
            print(yml)
        else:
            for name in yml:
                print(name)
    elif parser.operations[0] == vimanArgParser.vimanOperations.operations['version'][0]:
        print('{}-{}'.format(PROGRAM,VERSION))
    elif parser.operations[0] == vimanArgParser.vimanOperations.operations['help'][0]:
        vimanArgParser.vimanArgParser.printUsage()
        #parser.printUsage()
    else:
        print('error:invalid operation `{}`!'.format(parser.operations[0]),
                file=sys.stderr)
        return errno.EINVAL

    return 0

if '__main__' == __name__:
    sys.exit(main())




import re

from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

LONG_DESCRIPTION = open('README.rst').read()

def find_attr(attr, file_path):
    with open(file_path) as f:
        attr_file = f.read()
    attr_match = re.search(
            r"^{}\s*=\s*['\"]([^'\"]*)['\"]".format(attr),
            attr_file,
            re.M)

    if attr_match:
        return attr_match.group(1)

    raise RuntimeError("Unable to find version string.")

setup(name=find_attr('__program__', 'viman/__init__.py'),
        version=find_attr('__version__', 'viman/__init__.py'),
        description='The vim plugin manager based on git and pathogen!',
        long_description = LONG_DESCRIPTION,
        url='https://github.com/Shylock-Hg/viman.git',
        author='Shylock Hg',
        author_email='tcath2s@gmail.com',
        license='GPL-3.0',
        packages=['viman'],
        install_requires = ['pyyaml'],
        entry_points = {
            'console_scripts': ['viman=viman.viman:main'],
            },
        zip_safe=False)



#! /usr/bin/env python3

import getopt
import sys
import errno
import copy

class vimanOperations():
    ## (short,long,des)
    operations = {'sync':('S','sync','Synchronize vim plugin package!'),
                  'remove':('R','remove','Remove vim plugin package!'),
                  'upgrade':('U','upgrade','Upgrade vim plugin package!'),
                  'query':('Q','query','Query vim plugin package!'),
                  'version':('V','version','Show version of application!'),
                  'help':('h','help','Print help information!')}
    @staticmethod
    def getKey4Operation(operation:str):
        '''
        @brief get key by operation name(such as 'Q','sync')
        @param operation name
        @retval key of operations
        '''
        for kv in vimanOperations.operations.items():
            if operation in kv[1]:
                return kv[0]

    @staticmethod
    def getOperationsShort():
        return [operation[0] for operation in vimanOperations.operations.values() 
                if not '' == operation[0]]

    @staticmethod
    def getOperationsLong():
        return [operation[1] for operation in vimanOperations.operations.values() 
                if not '' == operation[1]]
    @staticmethod
    def getOperations():
        return [ops[0] for ops in list(vimanOperations.operations.values())] + [ops[1] for ops in list(vimanOperations.operations.values())]


class vimanOptions():

    ## (short,long,operations,des)
    options = {'sysupgrade':('u','sysupgrade','S','Upgrade all vim plugin packages!'),
            'file':('f','file','SRU','Specify plugins by yml file!'),
            'recursive':('r','recursive','U','Recursive upgrade vim plugin packages!'),
            'current':('c','current','S','Specify a current yml context string'),
            'name':('n','name','R','Specify the name of plugins to remove!')} 

    @staticmethod
    def getKey4Option(option:str):
        '''
        @brief get key by option name(such as 'u','sysupgrade')
        @param option name
        @retval key of options
        '''
        for kv in vimanOptions.options.items():
            if option in kv[1]:
                return kv[0]
    @staticmethod
    def getOptionShort():
        return [option[0] for option in vimanOptions.options.values() if not '' == option[0]]

    @staticmethod
    def getOptionLong():
        return [option[1] for option in vimanOptions.options.values() 
                if not '' == option[1]]
    
    @staticmethod
    def getOptions():
        return [opt[0] for opt in list(vimanOptions.options.values())] + [opt[1] for opt in list(vimanOptions.options.values())]


class vimanArgParser():
    '''
    @brief viman command line arguments parser
    @note viman command line synopsis `viman <operation> [options] [targets]`
    '''
    def __init__(self,args):
        '''
        @brief parse command line arguments
        @param args sys.argv[1:]
        @retval generated object
        '''
        '''
        options_short = [option[0] for option in vimanOptions.options.values() if not '' == option[0]] + [operation[0] for operation in vimanOperations.operations.values() if not '' == operation[0]]
        options_long = [option[1] for option in vimanOptions.options.values() if not '' == option[1]] + [operation[1] for operation in vimanOperations.operations.values() if not '' == operation[1]]
        options_short = ''.join(options_short)
        '''
        options_short = vimanOperations.getOperationsShort() + vimanOptions.getOptionShort()
        #print(options_short)
        options_short = ''.join(options_short)
        options_long  = vimanOperations.getOperationsLong()  + vimanOptions.getOptionLong()
        #print(options_long)
        
        opts,rest = getopt.getopt(args, options_short, options_long)

        self.targets = rest ## targets git repository url

        opts = map(lambda opt : opt[0].replace('-',''), opts)
        #print(list(opts))

        '''
        operations = [ops[0] for ops in list(vimanOperations.operations.values())] + [ops[1] for ops in list(vimanOperations.operations.values())]
        options = [opt[0] for opt in list(vimanOptions.options.values())] + [opt[1] for opt in list(vimanOptions.options.values())]
        '''
        operations = vimanOperations.getOperations()
        options = vimanOptions.getOptions()

        opts_operations = list(filter(lambda opt : opt in operations, copy.deepcopy(opts)))
        #print(opts_operations)
        opts_options = list(filter(lambda opt : opt in options, copy.deepcopy(opts)))

        # check operations
        if not 1 == len(opts_operations):
            print('error: only one operation may be used at a time!',file=sys.stderr)
            sys.exit(errno.EINVAL)

        # check options
        def check_option(option,operation):
            key_option = vimanOptions.getKey4Option(option)
            key_operation = vimanOperations.getKey4Operation(operation)
            if not vimanOperations.operations[key_operation][0] in vimanOptions.options[key_option][2]:
                print('error:invalid option `{}` for operation `{}`'.format(option,vimanOperations.operations[key_operation][0]),file=sys.stderr)
                sys.exit(errno.EINVAL)

        for opt in opts_options:
            check_option(opt,opts_operations[0])

        if len(opts_operations[0]) > 1:
            self.operations = vimanOperations.operations[opts_operations[0]][0] ## operations
        else:
            self.operations = opts_operations[0]

        opts_options_short = filter(lambda o : 1 == len(o), copy.deepcopy(opts_options))
        opts_options_long = filter(lambda o : 1 < len(o), copy.deepcopy(opts_options))
        opts_options_long = map(lambda l : vimanOptions.operations[ops][0], opts_options_long)
        opts_options = list(opts_options_short) + list(opts_options_long)
        self.options = opts_options ## options

    @staticmethod
    def printUsage():
        '''
        @brief print usage of application
        '''
        print('\nSynopsis of viman: viman <operation> [options...] [targets...]')
        for ops in vimanOperations.operations.values():
            print('\n\tOperation `-{},--{}` for {}'.format(ops[0],ops[1],ops[2]))
            for opt in vimanOptions.options.values():
                if ops[0] in opt[2]:
                    print('\t\tOption `-{},--{}` for {}'.format(opt[0],opt[1],opt[3]))


def _test():
    #print(vimanOperations.getOperations())
    #print(vimanOptions.getOptions())
    parser = vimanArgParser(sys.argv[1:])
    print(parser.operations)
    print(parser.options)
    print(parser.targets)

if '__main__' == __name__:
    _test()

